module APL.AST_Tests (tests) where

import APL.AST (Exp (..), printExp)
import Test.Tasty (TestTree, testGroup)
import Test.Tasty.HUnit (testCase, (@?=))

tests :: TestTree
tests =
  testGroup
    "Prettyprinting"
    [
      testCase "CstBool" $
        printExp (CstBool True) @?= "True",
      testCase "CstInt" $
        printExp (CstInt 500) @?= "500",
      testCase "Add" $
        printExp (Add (CstInt 1) (CstInt 9999)) @?= "(1 + 9999)",
      testCase "Mul" $
        printExp (Mul (CstInt 1) (CstInt 9999)) @?= "(1 * 9999)",
      testCase "Sub" $
        printExp (Sub (CstInt 1) (CstInt 9999)) @?= "(1 - 9999)",
      testCase "Div" $
        printExp (Div (CstInt 1) (CstInt 9999)) @?= "(1 / 9999)",
      testCase "Pow" $
        printExp (Pow (CstInt 1) (CstInt 9999)) @?= "(1 ** 9999)",
      testCase "If" $
        printExp (If (CstBool True) (CstInt 2) (CstInt 3))
        @?= "(if True then 2 else 3)",
      testCase "Var" $
        printExp (Var "missing")
        @?= "missing",
      testCase "Let" $
        printExp (Let "x" (CstInt 2) (Var "x"))
        @?= "(let x = 2 in x)",
      testCase "Lambda" $
        printExp (Lambda "x" (Var "x"))
        @?= "(\\x -> x)",
      testCase "Apply" $
        printExp (Apply (Lambda "x" (Var "x")) (CstInt 1))
        @?= "((\\x -> x) 1)",
      testCase "TryCatch" $
        printExp (TryCatch (CstInt 0) (CstInt 1))
        @?= "(try 0 catch 1)"
    ]
